<?php

namespace X_Currency\App\Controllers;

use X_Currency;
use X_Currency\App\Supports\Rate_Provider\Rate_Provider;
use X_Currency\App\Utils\Currency;

defined( 'ABSPATH' ) || exit;

class Currency_Controller
{
    /**
     * @param $data
     */
    public function create_currency( $data )
    {
        try {
            unset( $data['action'] );

            $code = strtoupper( str_replace( " ", "", sanitize_text_field( $data['code'] ) ) );

            $posts = get_posts( [
                'post_type'   => X_Currency::POST_TYPE,
                'post_status' => 'publish, draft',
                'numberposts' => 1,
                'meta_key'    => 'code',
                'meta_value'  => $code
            ] );

            if ( isset( $posts[0] ) ) {
                return [
                    'status'  => 'failed',
                    'message' => esc_html__( 'This currency code already exists', 'x-currency' )
                ];
            }

            $args = [
                'post_title'  => sanitize_text_field( $data['name'] ),
                'post_type'   => X_Currency::POST_TYPE,
                'post_status' => $data['active'] == true ? 'publish' : 'draft'
            ];

            $post_id = wp_insert_post( $args );

            update_post_meta( $post_id, 'code', $code );
            unset( $data['name'] );
            unset( $data['active'] );
            unset( $data['code'] );

            foreach ( $data as $key => $value ) {
                add_post_meta( $post_id, sanitize_key( $key ), map_deep( $value, 'sanitize_text_field' ) );
            }

            $sort_ids   = Currency::sort_ids();
            $sort_ids[] = $post_id;

            update_option( Currency::XCURRENCYSORTKEYS, serialize( $sort_ids ) );

            return [
                'message' => esc_html__( 'Currency created successfully!', 'x-currency' ),
                'data'    => ['currency_id' => $post_id],
                'status'  => 'success'
            ];
        } catch ( \Exception $e ) {
            return self::failed_response();
        }
    }

    /**
     * @param $data
     */
    public function update_currency( $data )
    {
        try {
            unset( $data['action'] );
            $post_id = sanitize_key( $data['id'] );

            $code     = strtoupper( str_replace( " ", "", sanitize_text_field( $data['code'] ) ) );
            $old_code = get_post_meta( $post_id, 'code', true );

            if ( $old_code != $code ) {

                $posts = get_posts( [
                    'post_type'   => X_Currency::POST_TYPE,
                    'post_status' => 'publish, draft',
                    'numberposts' => 1,
                    'meta_key'    => 'code',
                    'meta_value'  => $code
                ] );

                if ( isset( $posts[0] ) ) {
                    return [
                        'status'  => 'failed',
                        'message' => esc_html__( 'This currency code already exists', 'x-currency' )
                    ];
                }
            }

            $args = [
                'ID'          => $post_id,
                'post_title'  => sanitize_text_field( $data['name'] ),
                'post_status' => $data['active'] == true ? 'publish' : 'draft'
            ];

            update_post_meta( $post_id, 'code', $code );
            unset( $data['id'] );
            unset( $data['active'] );
            unset( $data['name'] );
            unset( $data['code'] );
            wp_update_post( $args );

            foreach ( $data as $key => $value ) {
                update_post_Meta( $post_id, sanitize_key( $key ), map_deep( $value, 'sanitize_text_field' ) );
            }
            return [
                'message' => esc_html__( 'Currency updated successfully!', 'x-currency' ),
                'status'  => 'success'
            ];
        } catch ( \Exception $e ) {
            return self::failed_response();
        }
    }

    /**
     * @param $params
     */
    public function currency_organizer( $data )
    {
        try {
            self::query( $data['keys'], $data['type'] );
            return [
                'message' => esc_html__( 'Currency organized successfully!', 'x-currency' ),
                'status'  => 'success'
            ];
        } catch ( \Exception $e ) {
            return self::failed_response();
        }
    }

    /**
     * @param $params
     */
    public function update_base_currency( $data )
    {
        if ( isset( $data['currency_id'] ) ) {
            update_option( Currency::XBASECURRENCY, sanitize_text_field( $data['currency_id'] ) );

            return [
                'status'  => 'success',
                'message' => esc_html__( 'Base currency updated successfully!', 'x-currency' )
            ];
        }

        return [
            'status'  => 'failed',
            'message' => esc_html__( 'Sorry, Something has wrong', 'x-currency' )
        ];
    }

    /**
     * @param $data
     */
    public function get_attachment( $data )
    {
        $ids        = $data['ids'];
        $attachment = [];

        if ( is_array( $ids ) ) {
            foreach ( $ids as $id ) {
                $attachment[] = wp_get_attachment_image_src( sanitize_key( $id ) );
            }
        }

        return [
            'message' => esc_html__( 'Attachment Retrieved Successfully!', 'x-currency' ),
            'data'    => $attachment,
            'status'  => 'success'
        ];
    }

    public function count_product()
    {
        return [
            'status' => 'success',
            'data'   => wp_count_posts( 'product' )
        ];
    }

    /**
     * @param $data
     * @return null
     */
    public function demo_currency_data( $data )
    {
        $rates = get_option( X_Currency::X_CURRENCY_RATES );

        if ( false === $rates ) {
            $base_currency_id   = get_option( Currency::XBASECURRENCY );
            $base_currency_code = get_post_meta( $base_currency_id, 'code', true );
            $rates              = serialize( Rate_Provider::exchange_base_currency( $base_currency_code, x_currency_get_json_file_content( X_CURRENCY_DIR . '/sample-data/rates.json' ) ) );
        }
        $rates   = unserialize( $rates );
        $symbols = x_currency_symbols();
        $data    = [];
        foreach ( get_woocommerce_currencies() as $code => $name ) {
            $data[$code] = [
                'name'               => $name,
                'code'               => $code,
                'symbol'             => isset( $symbols[$code] ) ? $symbols[$code] : '',
                'rate'               => isset( $rates['rates'][$code] ) ? $rates['rates'][$code] : '',
                'rate_type'          => 'auto',
                'extra_fee'          => '0', // must be string
                'extra_fee_type'     => 'fixed',
                'thousand_separator' => ',',
                'decimal_separator'  => '.',
                'decimal'            => 2,
                'position'           => 'left'
            ];
        }

        return [
            'data'   => $data,
            'status' => 'success'
        ];
    }

    public function currency_input_fields()
    {
        $payment_gateways    = [];
        $wc_payment_gateways = \WC_Payment_Gateways::instance();
        foreach ( $wc_payment_gateways->get_available_payment_gateways() as $key => $value ) {
            $payment_gateways[] = ['value' => $key, 'label' => $value->title];
        }

        return [
            'data'   => [
                'active'             => [
                    'label'   => esc_html__( 'Active', 'x-currency' ),
                    'type'    => 'XCurrencySwitch',
                    'default' => true
                ],
                'name'               => [
                    'label'       => esc_html__( 'Name', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter currency name', 'x-currency' ),
                    'type'        => 'XCurrencyText',
                    'required'    => true,
                    'message'     => esc_html__( 'Currency name field is required', 'x-currency' ),
                    'default'     => ''
                ],
                'code'               => [
                    'label'       => esc_html__( 'Code', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter currency code', 'x-currency' ),
                    'type'        => 'XCurrencyText',
                    'required'    => true,
                    'message'     => esc_html__( 'Currency code field is required', 'x-currency' ),
                    'default'     => ''
                ],
                'symbol'             => [
                    'label'       => esc_html__( 'Symbol', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter currency symbol', 'x-currency' ),
                    'type'        => 'XCurrencyText',
                    'required'    => true,
                    'message'     => esc_html__( 'Currency symbol field is required', 'x-currency' ),
                    'default'     => ''
                ],
                'flag'               => [
                    'label'    => esc_html__( 'Flag', 'x-currency' ),
                    'type'     => 'XCurrencyImage',
                    'required' => true,
                    'message'  => esc_html__( 'Currency flag field is required', 'x-currency' ),
                    'tooltips' => 'If you need to download flag, go ahead and download from overview page',
                    'default'  => ''
                ],
                'rate'               => [
                    'label'       => esc_html__( 'Rate( 1 Base Currency = ? )', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter currency rate', 'x-currency' ),
                    'type'        => 'XCurrencyNumber',
                    'required'    => true,
                    'message'     => esc_html__( 'Currency rate field is required', 'x-currency' ),
                    'default'     => ''
                ],
                'rate_type'          => [
                    'label'    => esc_html__( 'Rate Type', 'x-currency' ),
                    'type'     => 'XCurrencySelect',
                    'required' => true,
                    'options'  => [
                        ['value' => 'auto', 'label' => esc_html__( 'Auto', 'x-currency' )],
                        ['value' => 'fixed', 'label' => esc_html__( 'Fixed', 'x-currency' )]
                    ],
                    'message'  => esc_html__( 'Currency rate field is required', 'x-currency' ),
                    'default'  => 'auto'
                ],
                'extra_fee'          => [
                    'label'       => esc_html__( 'Extra fee for the total amount of the cart', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter extra fee', 'x-currency' ),
                    'type'        => 'XCurrencyNumber',
                    'default'     => ''
                ],
                'extra_fee_type'     => [
                    'label'   => esc_html__( 'Extra Fee Type', 'x-currency' ),
                    'type'    => 'XCurrencySelect',
                    'options' => [
                        ['value' => 'fixed', 'label' => esc_html__( 'Fixed', 'x-currency' )],
                        ['value' => 'percent', 'label' => esc_html__( '%', 'x-currency' )]
                    ],
                    'default' => 'fixed'
                ],
                'thousand_separator' => [
                    'label'       => esc_html__( 'Currency thousand separator', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter thousand separator', 'x-currency' ),
                    'type'        => 'XCurrencyText',
                    'required'    => true,
                    'message'     => esc_html__( 'Currency thousand separator field is required', 'x-currency' ),
                    'default'     => ','
                ],
                'decimal_separator'  => [
                    'label'       => esc_html__( 'Currency decimal separator', 'x-currency' ),
                    'placeholder' => esc_html__( 'Enter decimal separator', 'x-currency' ),
                    'type'        => 'XCurrencyText',
                    'required'    => true,
                    'message'     => esc_html__( 'Currency decimal separator field is required', 'x-currency' ),
                    'default'     => '.'
                ],
                'decimal'            => [
                    'type'        => 'XCurrencySelect',
                    'label'       => esc_html__( 'Currency decimal', 'x-currency' ),
                    'options'     => [
                        ['value' => 0, 'label' => 0],
                        ['value' => 1, 'label' => 1],
                        ['value' => 2, 'label' => 2],
                        ['value' => 3, 'label' => 3],
                        ['value' => 4, 'label' => 4],
                        ['value' => 5, 'label' => 5],
                        ['value' => 6, 'label' => 6],
                        ['value' => 7, 'label' => 7],
                        ['value' => 8, 'label' => 8],
                        ['value' => 9, 'label' => 9],
                        ['value' => 10, 'label' => 10]
                    ],
                    'default'     => 2,
                    'placeholder' => esc_html__( 'Select currency decimal', 'x-currency' ),
                    'required'    => true,
                    'message'     => esc_html__( 'Currency decimal field is required', 'x-currency' ),
                    'default'     => 2
                ],
                'position'           => [
                    'type'        => 'XCurrencySelect',
                    'label'       => esc_html__( 'Currency symbol position', 'x-currency' ),
                    'options'     => [
                        ['value' => 'left', 'label' => esc_html__( 'Left', 'x-currency' )],
                        ['value' => 'left_space', 'label' => esc_html__( 'Left space', 'x-currency' )],
                        ['value' => 'right', 'label' => esc_html__( 'Right', 'x-currency' )],
                        ['value' => 'right_space', 'label' => esc_html__( 'Right space', 'x-currency' )]
                    ],
                    'placeholder' => esc_html__( 'Select symbol position', 'x-currency' ),
                    'required'    => true,
                    'message'     => esc_html__( 'Currency symbol position field is required', 'x-currency' ),
                    'default'     => 'left_space'
                ],
                'payment_gateways'   => [
                    'type'     => 'XCurrencyMultiSelect',
                    'default'  => ['right'],
                    'label'    => esc_html__( 'Disable payment gateways for this currency', 'x-currency' ),
                    'options'  => $payment_gateways,
                    'tooltips' => esc_html__( 'Here WooCommerce payment gateways will appear.', 'x-currency' )
                ]
            ],
            'status' => 'success'
        ];
    }

    /**
     * @param $ids
     * @param $status
     * @return mixed
     */
    public static function query( $ids, $type, $sort = true )
    {
        switch ( $type ) {
            case 'active':
                foreach ( $ids as $id ) {
                    wp_update_post( ['ID' => $id, 'post_status' => 'publish'] );
                }
                break;
            case 'deactive':
                foreach ( $ids as $id ) {
                    wp_update_post( ['ID' => $id, 'post_status' => 'draft'] );
                }
                break;
            case 'delete':
                foreach ( $ids as $id ) {
                    wp_delete_post( $id, true );
                }
                if ( $sort ) {

                    $sort_ids     = array_diff( Currency::sort_ids(), $ids ); // remove ids form sortlist
                    $new_sort_ids = [];
                    foreach ( $sort_ids as $sort_id ) {
                        array_push( $new_sort_ids, $sort_id );
                    }

                    update_option( Currency::XCURRENCYSORTKEYS, serialize( $new_sort_ids ) );
                }
                break;
            case 'sort':
                update_option( Currency::XCURRENCYSORTKEYS, serialize( $ids ) );
        }
    }

    private static function failed_response()
    {
        return [
            'message' => esc_html__( 'Something is wrong', 'x-currency' ),
            'status'  => 'failed'
        ];
    }
}
