<?php

namespace X_Currency\App\Controllers;

use X_Currency;
use X_Currency\App\Supports\Rate_Provider\Rate_Provider;
use X_Currency\App\Supports\Schedule;
use X_Currency\App\Utils\Currency;
use X_Currency\App\Utils\Settings;

defined( 'ABSPATH' ) || exit;

class Settings_Controller
{
    /**
     * @return mixed
     */
    public function setting_inputs()
    {
        $data = Settings::settings_data();

        if ( empty( $data['base_currency'] ) ) {
            $currency_id = x_currency_get_currency_id_by_meta( 'code', get_woocommerce_currency() );
            if ( $currency_id !== false ) {
                $data['base_currency'] = $currency_id;
            }
        }

        $inputs = self::inputs();

        foreach ( $data as $key => $value ) {
            if ( isset( $inputs[$key] ) ) {
                if ( $value === 'true' ) {
                    $inputs[$key]['default'] = true;
                } elseif ( $value === 'false' ) {
                    $inputs[$key]['default'] = false;
                } else {
                    $inputs[$key]['default'] = $value;
                }
            }
        }
        return [
            'message' => esc_html__( 'Settings Input Retrieved Successfully!', 'x-currency' ),
            'data'    => $inputs,
            'status'  => 'success'
        ];
    }

    /**
     * @param $params
     * @return mixed
     */
    public function save_settings( $data )
    {
        try {
            unset( $data['action'] );
            $old_settings_data = Settings::settings_data();
            $old_base_currency = isset( $old_settings_data['base_currency'] ) ? $old_settings_data['base_currency'] : 0;

            if ( true == $data['currency_rate_auto_update'] ) {
                $transient_time = Schedule::instance()->calculate_transient_time( $data['rate_auto_update_time_type'], $data['rate_auto_update_time'] );
                set_transient( 'x-currency-currency-update', $transient_time, $transient_time );
            }

            if ( $old_base_currency != $data['base_currency'] ) {
                $code = get_post_meta( $data['base_currency'], 'code', true );

                if ( !empty( $code ) ) {
                    update_option( 'woocommerce_currency', $code );
                    update_option( Currency::XBASECURRENCY, $data['base_currency'] );

                    $get_rates = get_option( X_Currency::X_CURRENCY_RATES, [] );
                    if ( !is_array( $get_rates ) ) {
                        $get_rates = unserialize( $get_rates );
                    }

                    $rates = Rate_Provider::exchange_base_currency( $code, $get_rates );
                    Rate_Provider::exchange_all_currency( $rates['rates'] );
                }

                $code = get_post_meta( $data['base_currency'], 'code', true );
            }

            update_option( Settings::SETTINGS_OPTION_KEY, serialize( $data ) );

            return [
                'message' => esc_html__( 'Settings Data Updated Successfully!', 'x-currency' ),
                'data'    => [],
                'status'  => 'success'
            ];

        } catch ( \Exception $e ) {
            return Currency::failed_response();
        }
    }

    public static function inputs()
    {
        return [
            'user_welcome_currency_type'   => [
                'type'    => 'XCurrencySelect',
                'label'   => esc_html__( 'User welcome currency', 'x-currency' ),
                'options' => [
                    ['value' => 'fixed', 'label' => esc_html__( 'Fixed', 'x-currency' )],
                    ['value' => 'auto', 'label' => esc_html__( 'Base on geo ip', 'x-currency' )]
                ],
                'default' => ''
            ],
            'user_welcome_currency'        => [
                'type'          => 'XCurrencySelect',
                'option_source' => 'custom',
                'source_type'   => 'currency_list',
                'options'       => [],
                'label'         => esc_html__( 'Welcome currency', 'x-currency' ),
                'placeholder'   => esc_html__( 'Select a currency', 'x-currency' ),
                'required'      => true,
                'message'       => esc_html__( 'Default currency field is required', 'x-currency' ),
                'conditional'   => ['user_welcome_currency_type' => 'fixed'],
                'default'       => ''
            ],
            'base_currency'                => [
                'type'          => 'XCurrencySelect',
                'option_source' => 'custom',
                'source_type'   => 'currency_list',
                'options'       => [],
                'label'         => esc_html__( 'Base currency', 'x-currency' ),
                'placeholder'   => esc_html__( 'Select a currency', 'x-currency' ),
                'required'      => true,
                'message'       => esc_html__( 'Base currency field is required', 'x-currency' ),
                'default'       => get_option(Currency::XBASECURRENCY),
                'help'          => "You might need to refresh currency rate if you switch the base currency"
            ],
            'default_rate_provider'        => [
                'type'        => 'XCurrencySelect',
                'label'       => esc_html__( 'Currency rate aggregator', 'x-currency' ),
                'options'     => [
                    ['value' => 'fixer', 'label' => esc_html__( 'Fixer', 'x-currency' )],
                    ['value' => 'fixer-api-layer', 'label' => esc_html__( 'Fixer Api Layer', 'x-currency' )],
                    ['value' => 'currencyfreaks', 'label' => esc_html__( 'CurrencyFreaks', 'x-currency' )],
                    ['value' => 'openexchangerates', 'label' => esc_html__( 'OpenExchangeRates', 'x-currency' )]
                ],
                'isClearable' => true,
                'placeholder' => esc_html__( 'Select a aggregator', 'x-currency' ),
                'default'     => ''
            ],
            'fixer_api_token'              => [
                'label'       => esc_html__( 'Fixer api token', 'x-currency' ),
                'placeholder' => esc_html__( 'Enter fixer api token', 'x-currency' ),
                'type'        => 'XCurrencyText',
                'required'    => true,
                'message'     => esc_html__( 'Fixer api token field is required', 'x-currency' ),
                'conditional' => ['default_rate_provider' => 'fixer'],
                'help'        => '<a href="https://fixer.io/" target="_blank">Click here</a> for Fixer token',
                'default'     => ''
            ],
            'fixer-api-layer_api_token'    => [
                'label'       => esc_html__( 'Fixer api layer api token', 'x-currency' ),
                'placeholder' => esc_html__( 'Enter fixer api layer api token', 'x-currency' ),
                'type'        => 'XCurrencyText',
                'required'    => true,
                'message'     => esc_html__( 'Fixer api layer api token field is required', 'x-currency' ),
                'conditional' => ['default_rate_provider' => 'fixer-api-layer'],
                'help'        => '<a href="https://apilayer.com/marketplace/fixer-api" target="_blank">Click here</a> for Fixer api layer api token',
                'default'     => ''
            ],
            'currencyfreaks_api_token'     => [
                'label'       => esc_html__( 'CurrencyFreaks api token', 'x-currency' ),
                'placeholder' => esc_html__( 'Enter CurrencyFreaks api token', 'x-currency' ),
                'type'        => 'XCurrencyText',
                'required'    => true,
                'message'     => esc_html__( 'CurrencyFreaks api token field is required', 'x-currency' ),
                'conditional' => ['default_rate_provider' => 'currencyfreaks'],
                'help'        => '<a href="https://currencyfreaks.com/" target="_blank">Click here</a> for CurrencyFreaks token',
                'default'     => ''
            ],
            'openexchangerates_api_token'  => [
                'label'       => esc_html__( 'OpenExchangeRates api token', 'x-currency' ),
                'placeholder' => esc_html__( 'Enter OpenExchangeRates api token', 'x-currency' ),
                'type'        => 'XCurrencyText',
                'required'    => true,
                'message'     => esc_html__( 'OpenExchangeRates api token field is required', 'x-currency' ),
                'conditional' => ['default_rate_provider' => 'openexchangerates'],
                'help'        => '<a href="https://openexchangerates.org/" target="_blank">Click here</a> for OpenExchangeRates token',
                'default'     => ''
            ],
            'currency_rate_auto_update'    => [
                'label'   => esc_html__( 'Currency auto update', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false
            ],
            'rate_auto_update_time'        => [
                'name'        => '',
                'label'       => esc_html__( 'Currency rate auto update time', 'x-currency' ),
                'placeholder' => esc_html__( 'Enter currency rate auto update time', 'x-currency' ),
                'type'        => 'XCurrencyNumber',
                'required'    => true,
                'message'     => esc_html__( 'Currency auto update time field is required', 'x-currency' ),
                'conditional' => ['currency_rate_auto_update' => true],
                'default'     => 24
            ],
            'rate_auto_update_time_type'   => [
                'label'       => esc_html__( 'Currency rate auto update time type', 'x-currency' ),
                'placeholder' => esc_html__( 'Enter currency rate auto update time type', 'x-currency' ),
                'type'        => 'XCurrencySelect',
                'required'    => true,
                'message'     => esc_html__( 'Currency auto update time type field is required', 'x-currency' ),
                'conditional' => ['currency_rate_auto_update' => true],
                'options'     => [
                    ['value' => 'second', 'label' => esc_html__( 'Second', 'x-currency' )],
                    ['value' => 'minute', 'label' => esc_html__( 'Minute', 'x-currency' )],
                    ['value' => 'hour', 'label' => esc_html__( 'Hour', 'x-currency' )]
                ],
                'default'     => 'hour'
            ],
            'cart_total_extra_fee'         => [
                'label'   => esc_html__( 'Extra fee for the total amount of the cart', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'preview' => "<img src='" . esc_url( X_CURRENCY_URL . 'media/preview/extra-free-preview.webp' ) . "'/>",
                'default' => false
            ],
            'cart_total_extra_fee_message' => [
                'label'       => esc_html__( 'Show message of the cart total extra fee', 'x-currency' ),
                'type'        => 'XCurrencySwitch',
                'default'     => false,
                'conditional' => ['cart_total_extra_fee' => true]
            ],
            'specific_shipping_amount'     => [
                'label'   => esc_html__( 'Specific shipping amount for each currency', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false,
                'preview' => "<img src='" . esc_url( X_CURRENCY_URL . 'media/preview/shipping-amount-preview.webp' ) . "'/>"
            ],
            'specific_coupon_amount'       => [
                'label'   => esc_html__( 'Specific coupon amount for each currency', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false,
                'preview' => "<img src='" . esc_url( X_CURRENCY_URL . 'media/preview/coupon-amount-preview.webp' ) . "'/>"
            ],
            'specific_product_price'       => [
                'label'   => esc_html__( 'Specific product price for each currency', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false,
                'preview' => "<img src='" . esc_url( X_CURRENCY_URL . 'media/preview/product-price-preview.webp' ) . "'/>"
            ],
            'approximate_product_price'    => [
                'label'   => esc_html__( 'Approximate product price', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false,
                'preview' => "<img src='" . esc_url( X_CURRENCY_URL . 'media/preview/approximate-product-price.webp' ) . "'/>"
            ],
            'approximate_cart_price'       => [
                'label'   => esc_html__( 'Approximate cart and checkout total amount', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false
            ],
            'prices_without_cents'         => [
                'label'   => esc_html__( 'Prices without cents', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false
            ],
            'no_get_data_in_link'          => [
                'label'   => esc_html__( 'No GET data in link', 'x-currency' ),
                'type'    => 'XCurrencySwitch',
                'default' => false
            ]
        ];
    }

}
