<?php

namespace X_Currency\App\Service_Providers;

use X_Currency;
use X_Currency\App\Controllers\Currency_Controller;
use X_Currency\App\Supports\Rate_Provider\Rate_Provider;
use X_Currency\App\Utils\Currency;
use X_Currency\Bootstrap\Service_Provider;

defined( 'ABSPATH' ) || exit;

class Admin_Service_Provider implements Service_Provider
{
    public $page_url;

    public function boot()
    {
        if ( is_admin() ) {
            add_action( 'init', [$this, 'init'] );
        }
    }

    public function init()
    {
        $this->set_up_base_currency();
        $this->page_url = admin_url( 'admin.php?page=' . 'x-currency' );
        add_action( 'admin_menu', [$this, 'menu'] );
        add_action( 'admin_enqueue_scripts', [$this, 'load_admin_scripts'] );
        add_filter( 'plugin_action_links_x-currency/x-currency.php', [$this, 'plugin_action_links'] );
        add_filter( 'woocommerce_general_settings', [$this, 'woocommerce_general_settings'] );
        add_action( 'admin_notices', [ $this, 'action_admin_notices' ] );
    }

    public function action_admin_notices() : void {
        if(!function_exists('x_currency_pro_app')) {
        ?>
        <div class="notice" style="display:flex;font-size:18px;font-weight:500">
            <img style="width:90px; padding-top: 5px;" src="https://ps.w.org/x-currency/assets/icon-128x128.gif" alt="">
            <div style="padding-top: 14px;padding-left:15px">
                Great News - We are offering x-currency pro FREE for only our facebook community members. You will find X-Currency Pro update version in our Facebook community. So stay connected with us.
                <a class="button-primary" style="margin-top: 20px" href="https://www.facebook.com/groups/doatkolom">Join DoatKolom FaceBook Community</a>
            </div>
        </div>
        <?php
        }
    }

    /**
     * @param $data
     * @return mixed
     */
    public function woocommerce_general_settings( $data )
    {
        $general_remove_inputs = [
            'woocommerce_currency',
            'woocommerce_price_num_decimals',
            'woocommerce_currency_pos',
            'woocommerce_price_thousand_sep',
            'woocommerce_price_decimal_sep'
        ];

        foreach ( $data as $key => $value ) {

            if ( isset( $value['id'] ) ) {

                if ( in_array( $value['id'], $general_remove_inputs ) ) {
                    unset( $data[$key] );
                }

                if ( $value['id'] == 'pricing_options' ) {
                    $x_currency_logo    = '<h3 class="cursor-text" style="margin: 0px;color: var(--wp-admin-theme-color);"><i class="xc-icon-logo" style="margin-right: 10px;"></i>X-Currency</h3>';
                    $data[$key]['desc'] = $x_currency_logo . esc_html__( 'X-Currency plugin is activated. Please go to ', 'x-currency' ) . '<a href="' . $this->page_url . '#/global-settings' . '">' . esc_html__( 'X-Currency setting page', 'woo-multi-currency' ) . '</a>' . esc_html__( ' to change default currency.', 'x-currency' );
                }
            }
        }

        return $data;
    }

    public function set_up_base_currency()
    {
        if ( empty( get_option( Currency::XBASECURRENCY ) ) ) {

            $currency_code = get_woocommerce_currency();

            $currency = [];

            $symbol_code = get_woocommerce_currency_symbol( $currency_code );
            $symbol      = mb_convert_encoding( $symbol_code, 'UTF-8', 'HTML-ENTITIES' );

            $currency = [
                'active'             => true,
                'name'               => get_woocommerce_currencies()[$currency_code],
                'code'               => $currency_code,
                'symbol'             => $symbol,
                'rate'               => 1,
                'rate_type'          => 'fixed',
                'extra_fee'          => 0,
                'extra_fee_type'     => 'fixed',
                'thousand_separator' => wc_get_price_thousand_separator(),
                'decimal_separator'  => wc_get_price_decimal_separator(),
                'decimal'            => wc_get_price_decimals(),
                'position'           => get_option( 'woocommerce_currency_pos' )
            ];

            $response = ( new Currency_Controller )->create_currency( $currency );

            Rate_Provider::exchange_base_currency( $currency_code, x_currency_get_json_file_content( X_CURRENCY_DIR . '/sample-data/rates.json' ) );

            update_option( Currency::XBASECURRENCY, sanitize_text_field( $response['data']['currency_id'] ) );
        }
    }

    /**
     * Load Script for admin
     */
    public function load_admin_scripts()
    {
        wp_enqueue_media();

        $store_asset        = include_once( X_CURRENCY_DIR . '/build/store.asset.php' );
        $dashboard_assets   = include_once( X_CURRENCY_DIR . '/build/dashboard.asset.php' );

        array_push( $store_asset['dependencies'], 'lodash' );
        array_push( $store_asset['dependencies'], 'backbone' );
        array_push( $store_asset['dependencies'], 'jquery' );
        array_push( $dashboard_assets['dependencies'], 'x-currency-store' );
        
        /**
         * Load dashboard related assets that helps to use icons and admin menu 
         */ 
        wp_enqueue_script( 'x-currency-admin-menu', X_CURRENCY_URL . 'assets/js/admin-script.js', ['x-currency-store'], X_Currency::version() );
        wp_enqueue_style( 'x-currency-icon', X_CURRENCY_URL . 'media/font/style.css', [], X_Currency::version() );

        /**
         * Load dashboard related assets that helps to create dashboard 
         */ 
        wp_enqueue_script( 'x-currency-store', X_CURRENCY_URL . 'build/store.js', $store_asset['dependencies'], $store_asset['version'], true );
        do_action('x_currency_admin_script_before');

        wp_enqueue_style( 'x-currency-admin', X_CURRENCY_URL . 'build/dashboard.css', ['wp-components'], $dashboard_assets['version'] );
        wp_enqueue_script( 'x-currency-admin', X_CURRENCY_URL . 'build/dashboard.js', $dashboard_assets['dependencies'], $dashboard_assets['version'], true );

        wp_set_script_translations( 'x-currency-admin', 'x-currency' );
        wp_add_inline_style( 'x-currency-admin', "
        :root{
            --x-currency-bg-image: url(" . X_CURRENCY_URL . "media/common/dot-bg.png);
        }" );

        $base_currency = get_option( Currency::XBASECURRENCY );

        wp_localize_script( 'x-currency-store', 'x_currency', [
            'base_currency' => $base_currency
        ] );

        wp_localize_script( 'x-currency-admin', 'x_currency', [
            'version'       => X_Currency::version(),
            'apiUrl'        => get_rest_url( '', '' ),
            'media_url'     => X_CURRENCY_URL . 'media/',
            'prefix'        => X_Currency::POST_TYPE,
            'api_version'   => 'v1',
            'nonce'         => wp_create_nonce( 'wp_rest' ),
            'base_currency' => $base_currency
        ] );
    }

    /**
     * Add Admin menu
     */
    public function menu()
    {
        add_menu_page(
            esc_html__( 'X-Currency', 'x-currency' ),
            esc_html__( 'X-Currency', 'x-currency' ),
            'manage_options',
            'x-currency',
            [$this, 'content'],
            '', // logo
            '58.7'
        );

        add_submenu_page(
            'x-currency',
            esc_html__( 'Overview', 'x-currency' ),
            esc_html__( 'Overview', 'x-currency' ),
            'manage_options',
            esc_url( $this->page_url . '#/overview' )
        );

        add_submenu_page(
            'x-currency',
            esc_html__( 'Currencies', 'x-currency' ),
            esc_html__( 'Currencies', 'x-currency' ),
            'manage_options',
            esc_url( $this->page_url . '#/currencies' )
        );

        add_submenu_page(
            'x-currency',
            esc_html__( 'Switchers', 'x-currency' ),
            esc_html__( 'Switchers', 'x-currency' ),
            'manage_options',
            esc_url( $this->page_url . '#/switchers' )
        );

        add_submenu_page(
            'x-currency',
            esc_html__( 'Global Settings', 'x-currency' ),
            esc_html__( 'Global Settings', 'x-currency' ),
            'manage_options',
            esc_url( $this->page_url . '#/global-settings' )
        );

        add_submenu_page(
            'x-currency',
            esc_html__( 'Contact / Support', 'x-currency' ),
            esc_html__( 'Contact / Support', 'x-currency' ),
            'manage_options',
            esc_url( $this->page_url . '#/contact-support' )
        );

        // if(!class_exists('X_Currency_Pro')) {
        //     add_submenu_page(
        //         'x-currency',
        //         esc_html__( 'Go Po', 'x-currency' ),
        //         esc_html__( 'Contact / Support', 'x-currency' ),
        //         'manage_options',
        //         esc_url( $this->page_url . '#/contact-support' )
        //     );
        // }
    }

    /**
     * Admin dashboard
     */
    public function content()
    {
        include_once X_CURRENCY_DIR . 'resources/views/admin-screen.php';
    }

    /**
     * @param $links
     * @return mixed
     */
    public function plugin_action_links( $links )
    {
        $custom_links = [
            '<a href="https://doatkolom.com/contact" target="_blank" title="' . esc_attr__( 'Create support ticket', 'x-currency' ) . '">' . esc_html__( 'Get Support', 'x-currency' ) . '</a>',
            '<a href="https://www.demo.doatkolom.com/x-currency" target="_blank" title="' . esc_attr__( 'Demo', 'x-currency' ) . '">' . esc_html__( 'Demo', 'x-currency' ) . '</a>',
            '<a href="' . esc_url( $this->page_url . '#/currencies' ) . '" title="' . esc_attr__( 'Settings', 'x-currency' ) . '">' . esc_html__( 'Settings', 'x-currency' ) . '</a>'
        ];

        foreach ( $custom_links as $link ) {
            array_unshift( $links, $link );
        }

        return $links;
    }
}
